import pytest as pytest

from typer_router import Route

APP_1_IMPORT_PATH = "tests.input_files.app1"


@pytest.fixture
def app1_command1_route() -> Route:
    return Route.from_import_path("command1", is_dir=False)


@pytest.fixture
def app1_command1_1_route() -> Route:
    return Route.from_import_path("container1.command1_1", is_dir=False)


@pytest.fixture
def app1_command1_2_route() -> Route:
    return Route.from_import_path("container1.command1_2", is_dir=False)


@pytest.fixture
def app1_command2_1_1_route() -> Route:
    return Route.from_import_path("container2.container2_1.command2_1_1", is_dir=False)



from typing import Dict

import typer
from typer.models import TyperInfo


def assert_app1_typer_is_correct(typer_app: typer.Typer):
    assert typer_app.info.name == "app1"
    app_registered_groups = _registered_groups_dict(typer_app)
    app_registered_commands = _registered_commands_dict(typer_app)

    # Check container1.command1_1 and container1.command1_2
    assert "container1" in app_registered_groups
    container_1_group = app_registered_groups["container1"]
    container_1_commands = _registered_commands_dict(container_1_group.typer_instance)
    assert "command1_1" in container_1_commands
    assert "command1_2" in container_1_commands

    # Check container2.container2_1.command2_1_1
    assert "container2" in app_registered_groups
    container_2_group = app_registered_groups["container2"]
    container_2_registered_groups = _registered_groups_dict(
        container_2_group.typer_instance
    )
    assert "container2_1" in container_2_registered_groups
    container_2_1_group = container_2_registered_groups["container2_1"]
    container_2_1_commands = _registered_commands_dict(
        container_2_1_group.typer_instance
    )
    assert "command2_1_1" in container_2_1_commands

    # Check command1
    assert "command1" in app_registered_commands


def _registered_groups_dict(typer_app: typer.Typer) -> Dict[str, TyperInfo]:
    return {group.name: group for group in typer_app.registered_groups}


def _registered_commands_dict(typer_app: typer.Typer) -> Dict[str, TyperInfo]:
    return {command.name: command for command in typer_app.registered_commands}



import pytest
import typer

from tests.fixtures import APP_1_IMPORT_PATH
from tests.input_files import app1 as app1_module
from typer_router import Router


@pytest.fixture
def app1_typer_manually_constructed(app1_router: Router) -> typer.Typer:
    return app1_router.to_typer(name="app1")


@pytest.fixture
def app1_typer_from_app_import_path() -> typer.Typer:
    app1_router = Router.from_app_import_path(APP_1_IMPORT_PATH)
    return app1_router.to_typer(name="app1")


@pytest.fixture
def app1_typer_from_app_module() -> typer.Typer:
    app1_router = Router.from_app_module(app1_module)
    return app1_router.to_typer(name="app1")


@pytest.fixture(
    params=[
        app1_typer_manually_constructed,
        app1_typer_from_app_import_path,
        app1_typer_from_app_module,
    ]
)
def app1_typer(request) -> typer.Typer:
    return request.getfixturevalue(request.param.__name__)



from pathlib import Path
from typing import List

from pydantic import BaseModel

from typer_router.exc import NoParentRouteException

DEFAULT_FUNCTION_NAME = "main"


class Route(BaseModel):
    import_path: str
    name: str
    is_dir: bool
    function_name: str = DEFAULT_FUNCTION_NAME

    @property
    def parts(self) -> List[str]:
        return self.import_path.split(".")

    @property
    def subpaths(self) -> List[str]:
        parts = self.parts
        return [".".join(parts[:i]) for i in range(1, len(parts))]

    @property
    def subroutes(self) -> List["Route"]:
        return [
            Route.from_import_path(path, is_dir=True, function_name=self.function_name)
            for path in self.subpaths
        ]

    @property
    def parent(self) -> "Route":
        try:
            return self.subroutes[-1]
        except IndexError:
            raise NoParentRouteException(f"No parent route for {self.import_path}")

    @property
    def depth(self) -> int:
        return len(self.parts)

    @classmethod
    def from_file_path(
        cls, file_path: Path, function_name: str = DEFAULT_FUNCTION_NAME
    ) -> "Route":
        import_path = file_path.as_posix().replace("/", ".").replace(".py", "")
        name = file_path.stem
        return cls(
            import_path=import_path,
            name=name,
            is_dir=False,
            function_name=function_name,
        )

    @classmethod
    def from_import_path(
        cls, import_path: str, is_dir: bool, function_name: str = DEFAULT_FUNCTION_NAME
    ) -> "Route":
        name = import_path.split(".")[-1]
        return cls(
            import_path=import_path,
            name=name,
            is_dir=is_dir,
            function_name=function_name,
        )
